import 'dart:io';
import 'dart:convert';

import 'package:origami_endpoint/models/endpointConnection.dart';
import 'package:origami_endpoint/models/endpointErrorMessage.dart';
import 'package:origami_endpoint/models/uploadModel.dart';
import 'package:origami_endpoint/service/sessionManager/sessionManager.dart';
import 'package:origamicore/models/messageModel.dart';
import 'package:origamicore/routing/routeResponse.dart';
import 'package:origamicore/routing/router.dart';

import 'package:mime/mime.dart';
import 'package:uuid/uuid.dart';
class HttpIndex
{
  EndpointConnection connection = EndpointConnection();
  SessionManager sessionManager;
  HttpIndex({this.connection,this.sessionManager})
  {
    init();
  }
  dynamic init()async
  { 
    //var a=InternetAddress.loopbackIPv4;
    var bindAddress=connection.bindAddress??InternetAddress.loopbackIPv4; 
    var server = await HttpServer.bind(bindAddress, connection.protocol?.port??0);
    print (bindAddress);
    print("Search server is running on  'http://${server.address.address}:${connection.protocol?.port??0}/'");
     await for (var req in server) {
       accepRequest(req);
    } 
  }
  void accepRequest(HttpRequest  req)async
  {
    var strResponse='';
    req.response.headers.add('Access-Control-Allow-Origin', '*');
    req.response.headers.add('Access-Control-Allow-Methods', 'POST,GET,DELETE,PUT,OPTIONS');
    if(connection.allowHeader!=null)
    {
      req.response.headers.add('Access-Control-Allow-Headers', connection.allowHeader); 
      req.response.headers.add('Access-Control-Request-Headers', connection.allowHeader); 
      req.response.headers.add('Access-Control-Allow-Credentials', true); 
    } 
    if(req.method=='OPTIONS')
    {
      req.response.statusCode=200;
        req.response.write('{}');
        await req.response.close();
      return;
    }
       try{
          var resp= await sendRequest(req);
          if(resp.error!=null)
          {
            strResponse=json.encode(resp.error?.toJson()); 
            req.response.statusCode=500;
          }
          else
          {
            strResponse=json.encode(resp.response?.toJson()); 
            req.response.statusCode=200;

          }
       }catch(exp)
       {
          strResponse=exp.toString();
       }
       
      req.response.write(strResponse);
      await req.response.close();

  }
  Future<UploadModel> getUploadedFiles(HttpRequest req)async
  {
    
    var existUpload = connection.uploads.indexWhere((el) => el.url==req.uri.path);
    if(existUpload!=-1)
    {
      var upload=connection.uploads[existUpload];
      final uploadDirectory = upload.path;
      if(upload.maxSize!=null && upload.maxSize<req.contentLength)
      {
        throw 'Max file size is :${upload.maxSize.toString()} Byte';
      }
      // BytesBuilder  builder=await req.fold(new BytesBuilder(), (builder, data) => builder..add(data));

      // var file = new File('$uploadDirectory/abc.jpg');
      // var f =await  file.writeAsBytes(builder.takeBytes(), mode: FileMode.WRITE);







      var dataBytes = <int>[];
      
      await for (var data in req) {
        dataBytes.addAll(data.toList());
      }
      
      var boundary = req.headers.contentType.parameters['boundary'];
      final transformer = MimeMultipartTransformer(boundary);
      final bodyStream = Stream.fromIterable([dataBytes]);
      final parts = await transformer.bind(bodyStream).toList();
      for (var part in parts) {
        print(part.headers);
        final contentDisposition = part.headers['content-disposition'];
        final filename = RegExp(r''+upload.name+'="([^"]*)"')
            .firstMatch(contentDisposition)
            .group(1);
        var ext=filename.split('.');
        var exportName=Uuid().v4()+'.'+ext[ext.length-1];    
        final content = await part.toList();
        if (!Directory(uploadDirectory).existsSync()) {
          await Directory(uploadDirectory).create();
        }
        await File('$uploadDirectory/$exportName').writeAsBytes(content[0]);
        return UploadModel(path:'$uploadDirectory/$exportName',name: upload.name );
      }
    }
    return null;
  }
  Future<RouteResponse> sendRequest(HttpRequest request) async
  {
    var path=request.uri.pathSegments;
    if(path.length<2)
    {
      return  RouteResponse(error: EndpointErrorMessage.connectionNotFound);
    }
    var body={};
    var upload=await getUploadedFiles(request);
    if(request.method=='POST')
    {
      try{
        var dt =await  Utf8Codec().decodeStream(request);
        body = json.decode(dt);

      }catch(exp){}
    }
    request.uri.queryParameters.forEach((key, value) { 
      body[key]=value;
    }); 
    dynamic internalData={};
    if(upload!=null)
    {
      internalData[upload.name]=upload.path;
    }
    var message= MessageModel(data: body,internalData:internalData );
    if(sessionManager!=null && request.headers['authorization']!=null)
    {
      message.session=await sessionManager.getSession(request.headers['authorization'][0]);
    }
    // var dt =await UTF8.decodeStream(request);
    var resp=await Router.runExternal(path[0], path[1],message);
    
    if(resp.session!=null)
    {
      resp.response.session=await sessionManager.setSession('token', resp.session);
    }
    return resp;
  }
}